import { isPlatformServer, DOCUMENT } from '@angular/common';
import * as i0 from '@angular/core';
import { EventEmitter, afterNextRender, PLATFORM_ID, Component, ChangeDetectionStrategy, Inject, Input, Output } from '@angular/core';

class ScriptOptimizerComponent {
    document;
    platformId;
    injector;
    constructor(document, platformId, injector) {
        this.document = document;
        this.platformId = platformId;
        this.injector = injector;
    }
    src;
    scriptContent;
    appendTo = 'head';
    loadStrategy = 'lazy';
    renderStrategy = 'server';
    contentType = 'text/javascript';
    integrity;
    origin;
    onLoad = new EventEmitter();
    worker;
    scriptElem;
    ngOnInit() {
        try {
            if (isPlatformServer(this.platformId) && this.renderStrategy == 'server') {
                this.addScriptToHead();
            }
            if (this.renderStrategy == 'client') {
                afterNextRender(() => {
                    this.addScriptToHead();
                }, { injector: this.injector });
            }
        }
        catch (error) {
            console.error(error);
        }
    }
    addScriptToHead() {
        if (!this.scriptContent && !this.src) {
            throw new Error('ScriptOptimizerComponent: Either `src` or `scriptContent` must be provided.');
        }
        if (this.scriptContent && this.src) {
            throw new Error('ScriptOptimizerComponent: Both `src` or `scriptContent` cannot be provided at the same time.');
        }
        if (this.loadStrategy == 'worker') {
            if (typeof Worker !== 'undefined') {
                if (this.scriptContent) {
                    this.createAndExecuteWorker(this.scriptContent);
                    return;
                }
                throw new Error('ScriptOptimizerComponent: `scriptContent` must be provided.');
            }
            return;
        }
        this.scriptElem = this.document.createElement('script');
        this.scriptElem.type = this.contentType;
        this.scriptElem.integrity = this.integrity;
        this.scriptElem.crossOrigin = this.origin;
        if (this.src) {
            this.scriptElem.src = this.src;
        }
        if (this.scriptContent) {
            this.scriptElem.text = this.scriptContent;
        }
        this.scriptElem.onload = () => {
            this.onLoad.emit();
        };
        if (this.loadStrategy == 'eager') {
            this.scriptElem.defer = false;
            this.scriptElem.async = true;
            this.appendScripts();
        }
        if (this.loadStrategy == 'lazy') {
            this.scriptElem.defer = true;
            this.scriptElem.async = true;
            this.appendScripts();
        }
        if (this.loadStrategy == 'idle') {
            this.scriptElem.defer = true;
            this.scriptElem.async = true;
            if (typeof window != 'undefined' && 'requestIdleCallback' in window) {
                this.appendScripts();
            }
        }
    }
    createAndExecuteWorker(scriptContent) {
        const blob = new Blob([scriptContent], { type: 'application/javascript' });
        const workerUrl = URL.createObjectURL(blob);
        this.worker = new Worker(workerUrl);
        this.worker.onmessage = (event) => {
            console.log('Message from worker:', event.data);
            this.onLoad.emit();
        };
        this.worker.onerror = (error) => {
            console.error('Error in worker:', error.message);
        };
    }
    appendScripts() {
        if (this.appendTo == 'head') {
            this.document.head.appendChild(this.scriptElem);
        }
        if (this.appendTo == 'body') {
            this.document.body.appendChild(this.scriptElem);
        }
    }
    ngOnDestroy() {
        if (this.scriptElem && typeof document != 'undefined') {
            document?.body?.removeChild(this.scriptElem);
            document?.head?.removeChild(this.scriptElem);
            this.worker?.terminate();
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: ScriptOptimizerComponent, deps: [{ token: DOCUMENT }, { token: PLATFORM_ID }, { token: i0.Injector }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.0.5", type: ScriptOptimizerComponent, isStandalone: true, selector: "ngx-script-optimizer", inputs: { src: "src", scriptContent: "scriptContent", appendTo: "appendTo", loadStrategy: "loadStrategy", renderStrategy: "renderStrategy", contentType: "contentType", integrity: "integrity", origin: "origin" }, outputs: { onLoad: "onLoad" }, ngImport: i0, template: ``, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.0.5", ngImport: i0, type: ScriptOptimizerComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngx-script-optimizer',
                    imports: [],
                    template: ``,
                    changeDetection: ChangeDetectionStrategy.OnPush
                }]
        }], ctorParameters: () => [{ type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }, { type: i0.Injector }], propDecorators: { src: [{
                type: Input
            }], scriptContent: [{
                type: Input
            }], appendTo: [{
                type: Input
            }], loadStrategy: [{
                type: Input
            }], renderStrategy: [{
                type: Input
            }], contentType: [{
                type: Input
            }], integrity: [{
                type: Input
            }], origin: [{
                type: Input
            }], onLoad: [{
                type: Output
            }] } });

/*
 * Public API Surface of script-optimizer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ScriptOptimizerComponent };
//# sourceMappingURL=ngx-script-optimizer.mjs.map
